@@ -1,7 +1,6 @@
+require 'mechanize/version'
 require 'fileutils'
 require 'forwardable'
-require 'iconv' if RUBY_VERSION < '1.9.2'
-require 'mime/types'
 require 'mutex_m'
 require 'net/http/digest_auth'
 require 'net/http/persistent'
@@ -70,11 +69,6 @@ require 'zlib'
 
 class Mechanize
 
-  ##
-  # The version of Mechanize you are using.
-
-  VERSION = '2.7.3'
-
   ##
   # Base mechanize error class
 
@@ -92,22 +86,26 @@ class Mechanize
   # description in parenthesis is for informative purposes and is not part of
   # the alias name.
   #
-  # * Linux Firefox (3.6.1)
+  # * Linux Firefox (43.0 on Ubuntu Linux)
   # * Linux Konqueror (3)
   # * Linux Mozilla
-  # * Mac Firefox (3.6)
+  # * Mac Firefox (43.0)
   # * Mac Mozilla
-  # * Mac Safari (5)
+  # * Mac Safari (9.0 on OS X 10.11.2)
   # * Mac Safari 4
   # * Mechanize (default)
   # * Windows IE 6
   # * Windows IE 7
   # * Windows IE 8
   # * Windows IE 9
+  # * Windows IE 10 (Windows 8 64bit)
+  # * Windows IE 11 (Windows 8.1 64bit)
+  # * Windows Edge
   # * Windows Mozilla
-  # * iPhone (3.0)
-  # * iPad
-  # * Android (Motorola Xoom)
+  # * Windows Firefox (43.0)
+  # * iPhone (iOS 9.1)
+  # * iPad (iOS 9.1)
+  # * Android (5.1.1)
   #
   # Example:
   #
@@ -116,26 +114,37 @@ class Mechanize
 
   AGENT_ALIASES = {
     'Mechanize' => "Mechanize/#{VERSION} Ruby/#{ruby_version} (http://github.com/sparklemotion/mechanize/)",
-    'Linux Firefox' => 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.1) Gecko/20100122 firefox/3.6.1',
+    'Linux Firefox' => 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:43.0) Gecko/20100101 Firefox/43.0',
     'Linux Konqueror' => 'Mozilla/5.0 (compatible; Konqueror/3; Linux)',
     'Linux Mozilla' => 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.4) Gecko/20030624',
-    'Mac Firefox' => 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.6; en-US; rv:1.9.2) Gecko/20100115 Firefox/3.6',
+    'Mac Firefox' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.11; rv:43.0) Gecko/20100101 Firefox/43.0',
     'Mac Mozilla' => 'Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.4a) Gecko/20030401',
     'Mac Safari 4' => 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; de-at) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10',
-    'Mac Safari' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/534.51.22 (KHTML, like Gecko) Version/5.1.1 Safari/534.51.22',
-    'Windows Chrome' => 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.32 Safari/537.36',
+    'Mac Safari' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/601.3.9 (KHTML, like Gecko) Version/9.0.2 Safari/601.3.9',
+    'Windows Chrome' => 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.125 Safari/537.36',
     'Windows IE 6' => 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)',
     'Windows IE 7' => 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)',
     'Windows IE 8' => 'Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727)',
     'Windows IE 9' => 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)',
+    'Windows IE 10' => 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)',
+    'Windows IE 11' => 'Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko',
+    'Windows Edge' => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Safari/537.36 Edge/13.10586',
     'Windows Mozilla' => 'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.4b) Gecko/20030516 Mozilla Firebird/0.6',
-    'iPhone' => 'Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C28 Safari/419.3',
-    'iPad' => 'Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10',
-    'Android' => 'Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13'
+    'Windows Firefox' => 'Mozilla/5.0 (Windows NT 6.3; WOW64; rv:43.0) Gecko/20100101 Firefox/43.0',
+    'iPhone' => 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B5110e Safari/601.1',
+    'iPad' => 'Mozilla/5.0 (iPad; CPU OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1',
+    'Android' => 'Mozilla/5.0 (Linux; Android 5.1.1; Nexus 7 Build/LMY47V) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.76 Safari/537.36',
   }
 
-  AGENT_ALIASES['Mac FireFox'] = AGENT_ALIASES['Mac Firefox']
-  AGENT_ALIASES['Linux FireFox'] = AGENT_ALIASES['Linux Firefox']
+  AGENT_ALIASES.default_proc = proc { |hash, key|
+    case key
+    when /FireFox/
+      if ua = hash[nkey = key.sub(/FireFox/, 'Firefox')]
+        warn "Mechanize#user_agent_alias: #{key.inspect} should be spelled as #{nkey.inspect}"
+        ua
+      end
+    end
+  }
 
   def self.inherited(child) # :nodoc:
     child.html_parser = html_parser
@@ -342,12 +351,21 @@ class Mechanize
         click real_link
       else
         button = nil
+        # Note that this will not work if we have since navigated to a different page.
+        # Should rather make each button aware of its parent form.
         form = page.forms.find do |f|
           button = f.button_with(:value => link)
           button.is_a? Form::Submit
         end
         submit form, button if form
       end
+    when Form::Submit, Form::ImageButton then
+      # Note that this will not work if we have since navigated to a different page.
+      # Should rather make each button aware of its parent form.
+      form = page.forms.find do |f|
+        f.buttons.include?(link)
+      end
+      submit form, link if form
     else
       referer = current_page()
       href = link.respond_to?(:href) ? link.href :
@@ -400,6 +418,9 @@ class Mechanize
   ##
   # DELETE +uri+ with +query_params+, and setting +headers+:
   #
+  # +query_params+ is formatted into a query string using
+  # Mechanize::Util.build_query_string, which see.
+  #
   #   delete('http://example/', {'q' => 'foo'}, {})
 
   def delete(uri, query_params = {}, headers = {})
@@ -413,6 +434,9 @@ class Mechanize
   # +headers+.
   #
   # The +referer+ may be a URI or a page.
+  #
+  # +parameters+ is formatted into a query string using
+  # Mechanize::Util.build_query_string, which see.
 
   def get(uri, parameters = [], referer = nil, headers = {})
     method = :get
@@ -453,6 +477,9 @@ class Mechanize
   ##
   # HEAD +uri+ with +query_params+ and +headers+:
   #
+  # +query_params+ is formatted into a query string using
+  # Mechanize::Util.build_query_string, which see.
+  #
   #   head('http://example/', {'q' => 'foo'}, {})
 
   def head(uri, query_params = {}, headers = {})
@@ -464,9 +491,13 @@ class Mechanize
   end
 
   ##
-  # POST to the given +uri+ with the given +query+.  The query is specified by
-  # either a string, or a list of key-value pairs represented by a hash or an
-  # array of arrays.
+  # POST to the given +uri+ with the given +query+.
+  #
+  # +query+ is processed using Mechanize::Util.each_parameter (which
+  # see), and then encoded into an entity body.  If any IO/FileUpload
+  # object is specified as a field value the "enctype" will be
+  # multipart/form-data, or application/x-www-form-urlencoded
+  # otherwise.
   #
   # Examples:
   #   agent.post 'http://example.com/', "foo" => "bar"
@@ -489,12 +520,15 @@ class Mechanize
 
     form = Form.new(node)
 
-    query.each { |k, v|
+    Mechanize::Util.each_parameter(query) { |k, v|
       if v.is_a?(IO)
         form.enctype = 'multipart/form-data'
         ul = Form::FileUpload.new({'name' => k.to_s},::File.basename(v.path))
         ul.file_data = v.read
         form.file_uploads << ul
+      elsif v.is_a?(Form::FileUpload)
+        form.enctype = 'multipart/form-data'
+        form.file_uploads << v
       else
         form.fields << Form::Field.new({'name' => k.to_s},v)
       end
@@ -518,6 +552,8 @@ class Mechanize
   def request_with_entity(verb, uri, entity, headers = {})
     cur_page = current_page || Page.new
 
+    log.debug("query: #{ entity.inspect }") if log
+
     headers = {
       'Content-Type' => 'application/octet-stream',
       'Content-Length' => entity.size.to_s,
@@ -934,6 +970,12 @@ Use of #auth and #basic_auth are deprecated due to a security vulnerability.
     @agent.redirection_limit = limit
   end
 
+  ##
+  # Resolve the full path of a link / uri
+  def resolve link
+    @agent.resolve link
+  end
+
   ##
   # A hash of custom request headers that will be sent on every request
 
@@ -1138,19 +1180,19 @@ Use of #auth and #basic_auth are deprecated due to a security vulnerability.
   end
 
   ##
-  # SSL version to use.  Ruby 1.9 and newer only.
+  # SSL version to use.
 
   def ssl_version
     @agent.ssl_version
-  end if RUBY_VERSION > '1.9'
+  end
 
   ##
   # Sets the SSL version to use to +version+ without client/server
-  # negotiation.  Ruby 1.9 and newer only.
+  # negotiation.
 
   def ssl_version= ssl_version
     @agent.ssl_version = ssl_version
-  end if RUBY_VERSION > '1.9'
+  end
 
   ##
   # A callback for additional certificate verification.  See
@@ -1316,7 +1358,6 @@ require 'mechanize/http/content_disposition_parser'
 require 'mechanize/http/www_authenticate_parser'
 require 'mechanize/image'
 require 'mechanize/page'
-require 'mechanize/monkey_patch'
 require 'mechanize/pluggable_parsers'
 require 'mechanize/redirect_limit_reached_error'
 require 'mechanize/redirect_not_get_or_head_error'
