@@ -51,6 +51,7 @@ class Mechanize::Download
 
   ##
   # Saves a copy of the body_io to +filename+
+  # returns the filename
 
   def save filename = nil
     filename = find_free_name filename
@@ -63,8 +64,10 @@ class Mechanize::Download
   # Use this method to save the content of body_io to +filename+.
   # This method will overwrite any existing filename that exists with the
   # same name.
+  # returns the filename
 
   def save! filename = nil
+    filename ||= @filename
     dirname = File.dirname filename
     FileUtils.mkdir_p dirname
 
@@ -73,6 +76,8 @@ class Mechanize::Download
         io.write @body_io.read 16384
       end
     end
+
+    filename
   end
 
 end
