@@ -64,18 +64,18 @@ class Mechanize::Form::MultiSelectList < Mechanize::Form::Field
   # Select no options
   def select_none
     @value = []
-    options.each { |o| o.untick }
+    options.each(&:untick)
   end
 
   # Select all options
   def select_all
     @value = []
-    options.each { |o| o.tick }
+    options.each(&:tick)
   end
 
   # Get a list of all selected options
   def selected_options
-    @options.find_all { |o| o.selected? }
+    @options.find_all(&:selected?)
   end
 
   def value=(values)
@@ -91,10 +91,7 @@ class Mechanize::Form::MultiSelectList < Mechanize::Form::Field
   end
 
   def value
-    value = []
-    value.concat @value
-    value.concat selected_options.map { |o| o.value }
-    value
+    @value + selected_options.map(&:value)
   end
 
 end
