@@ -42,7 +42,7 @@ class Mechanize::Page < Mechanize::File
     if body
       # Force the encoding to be 8BIT so we can perform regular expressions.
       # We'll set it to the detected encoding later
-      body.force_encoding 'ASCII-8BIT' if body.respond_to? :force_encoding
+      body.force_encoding(Encoding::ASCII_8BIT)
 
       @encodings.concat self.class.meta_charset body
 
@@ -92,7 +92,9 @@ class Mechanize::Page < Mechanize::File
   end
 
   def encoding
-    parser.respond_to?(:encoding) ? parser.encoding : nil
+    parser.encoding
+  rescue NoMethodError
+    nil
   end
 
   # Return whether parser result has errors related to encoding or not.
@@ -109,15 +111,17 @@ class Mechanize::Page < Mechanize::File
 
   def parser
     return @parser if @parser
-    return nil unless @body
+    return unless @body
 
-    if @encoding then
-      @parser = @mech.html_parser.parse html_body, nil, @encoding
-    elsif mech.force_default_encoding then
-      @parser = @mech.html_parser.parse html_body, nil, @mech.default_encoding
+    url = @uri && @uri.to_s
+
+    if @encoding
+      @parser = mech.html_parser.parse html_body, url, @encoding
+    elsif mech.force_default_encoding
+      @parser = mech.html_parser.parse html_body, url, @mech.default_encoding
     else
       @encodings.reverse_each do |encoding|
-        @parser = @mech.html_parser.parse html_body, nil, encoding
+        @parser = mech.html_parser.parse html_body, url, encoding
 
         break unless encoding_error? @parser
       end
@@ -192,39 +196,71 @@ class Mechanize::Page < Mechanize::File
   ##
   # :method: search
   #
-  # Search for +paths+ in the page using Nokogiri's #search.  The +paths+ can
-  # be XPath or CSS and an optional Hash of namespaces may be appended.
+  # Shorthand for +parser.search+.
   #
-  # See Nokogiri::XML::Node#search for further details.
+  # See Nokogiri::XML::Node#search for details.
 
-  def_delegator :parser, :search, :search
+  ##
+  # :method: css
+  #
+  # Shorthand for +parser.css+.
+  #
+  # See also Nokogiri::XML::Node#css for details.
 
-  alias / search
+  ##
+  # :method: xpath
+  #
+  # Shorthand for +parser.xpath+.
+  #
+  # See also Nokogiri::XML::Node#xpath for details.
 
   ##
   # :method: at
   #
-  # Search through the page for +path+ under +namespace+ using Nokogiri's #at.
-  # The +path+ may be either a CSS or XPath expression.
+  # Shorthand for +parser.at+.
   #
-  # See also Nokogiri::XML::Node#at
+  # See also Nokogiri::XML::Node#at for details.
 
-  def_delegator :parser, :at, :at
+  ##
+  # :method: at_css
+  #
+  # Shorthand for +parser.at_css+.
+  #
+  # See also Nokogiri::XML::Node#at_css for details.
+
+  ##
+  # :method: at_xpath
+  #
+  # Shorthand for +parser.at_xpath+.
+  #
+  # See also Nokogiri::XML::Node#at_xpath for details.
+
+  def_delegators :parser, :search, :css, :xpath, :at, :at_css, :at_xpath
+
+  alias / search
+  alias % at
 
   ##
   # :method: form_with
   #
   # :call-seq:
   #   form_with(criteria)
+  #   form_with(criteria) { |form| ... }
   #
-  # Find a single form matching +criteria+.
-  # Example:
-  #   page.form_with(:action => '/post/login.php') do |f|
+  # Find a single form matching +criteria+.  See +forms_with+ for
+  # details of +criteria+.
+  #
+  # Examples:
+  #   page.form_with(action: '/post/login.php') do |f|
   #     ...
   #   end
 
   ##
-  # :mehtod: form_with!(criteria)
+  # :method: form_with!(criteria)
+  #
+  # :call-seq:
+  #   form_with!(criteria)
+  #   form_with!(criteria) { |form| ... }
   #
   # Same as +form_with+ but raises an ElementNotFoundError if no button matches
   # +criteria+
@@ -232,11 +268,33 @@ class Mechanize::Page < Mechanize::File
   ##
   # :method: forms_with
   #
-  # :call-seq: forms_with(criteria)
+  # :call-seq:
+  #   forms_with(name)
+  #   forms_with(name: name_matcher, id: id_matcher, class: class_matcher,
+  #              search: search_expression, xpath: xpath_expression, css: css_expression,
+  #              action: action_matcher, ...)
+  #
+  # Find all forms form matching criteria.  If a string is given, it
+  # is taken as a name attribute value.  If a hash is given, forms
+  # are narrowed by the key-value pairs as follows.
+  #
+  # :id, :dom_id: selects forms with a #dom_id value that matches this
+  # value.
+  #
+  # :class, :dom_class: selects forms with a #dom_class value that
+  # matches this value.
+  #
+  # :search: only selects forms matching this selector expression.
+  #
+  # :xpath: only selects forms matching this XPath expression.
+  #
+  # :css: only selects forms matching this CSS selector expression.
+  #
+  # :action, :method, etc.: narrows forms by a given attribute value
+  # using the === operator.
   #
-  # Find all forms form matching +criteria+.
   # Example:
-  #   page.forms_with(:action => '/post/login.php').each do |f|
+  #   page.forms_with(css: '#content table.login_box form', method: /\APOST\z/i, ).each do |f|
   #     ...
   #   end
 
@@ -245,14 +303,22 @@ class Mechanize::Page < Mechanize::File
   ##
   # :method: link_with
   #
-  # :call-seq: link_with(criteria)
+  # :call-seq:
+  #   link_with(criteria)
+  #   link_with(criteria) { |link| ... }
+  #
+  # Find a single link matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "link(s)".
   #
-  # Find a single link matching +criteria+.
   # Example:
-  #   page.link_with(:href => /foo/).click
+  #   page.link_with(href: /foo/).click
 
   ##
-  # :mehtod: link_with!(criteria)
+  # :method: link_with!
+  #
+  # :call-seq:
+  #   link_with!(criteria)
+  #   link_with!(criteria) { |link| ... }
   #
   # Same as +link_with+ but raises an ElementNotFoundError if no button matches
   # +criteria+
@@ -263,9 +329,11 @@ class Mechanize::Page < Mechanize::File
   # :call-seq:
   #   links_with(criteria)
   #
-  # Find all links matching +criteria+.
+  # Find all links matching +criteria+.  See +forms_with+ for details
+  # of +criteria+, where for "form(s)" read "link(s)".
+  #
   # Example:
-  #   page.links_with(:href => /foo/).each do |link|
+  #   page.links_with(href: /foo/).each do |link|
   #     puts link.href
   #   end
 
@@ -274,14 +342,22 @@ class Mechanize::Page < Mechanize::File
   ##
   # :method: base_with
   #
-  # :call-seq: base_with(criteria)
+  # :call-seq:
+  #   base_with(criteria)
+  #   base_with(criteria) { |base| ... }
+  #
+  # Find a single base tag matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "base tag(s)".
   #
-  # Find a single base tag matching +criteria+.
   # Example:
-  #   page.base_with(:href => /foo/).click
+  #   page.base_with(href: /foo/).click
 
   ##
-  # :mehtod: base_with!(criteria)
+  # :method: base_with!(criteria)
+  #
+  # :call-seq:
+  #   base_with!(criteria)
+  #   base_with!(criteria) { |base| ... }
   #
   # Same as +base_with+ but raises an ElementNotFoundError if no button matches
   # +criteria+
@@ -291,9 +367,11 @@ class Mechanize::Page < Mechanize::File
   #
   # :call-seq: bases_with(criteria)
   #
-  # Find all base tags matching +criteria+.
+  # Find all base tags matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "base tag(s)".
+  #
   # Example:
-  #   page.bases_with(:href => /foo/).each do |base|
+  #   page.bases_with(href: /foo/).each do |base|
   #     puts base.href
   #   end
 
@@ -302,14 +380,22 @@ class Mechanize::Page < Mechanize::File
   ##
   # :method: frame_with
   #
-  # :call-seq: frame_with(criteria)
+  # :call-seq:
+  #   frame_with(criteria)
+  #   frame_with(criteria) { |frame| ... }
+  #
+  # Find a single frame tag matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "frame tag(s)".
   #
-  # Find a single frame tag matching +criteria+.
   # Example:
-  #   page.frame_with(:src => /foo/).click
+  #   page.frame_with(src: /foo/).click
 
   ##
-  # :mehtod: frame_with!(criteria)
+  # :method: frame_with!
+  #
+  # :call-seq:
+  #   frame_with!(criteria)
+  #   frame_with!(criteria) { |frame| ... }
   #
   # Same as +frame_with+ but raises an ElementNotFoundError if no button matches
   # +criteria+
@@ -319,9 +405,11 @@ class Mechanize::Page < Mechanize::File
   #
   # :call-seq: frames_with(criteria)
   #
-  # Find all frame tags matching +criteria+.
+  # Find all frame tags matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "frame tag(s)".
+  #
   # Example:
-  #   page.frames_with(:src => /foo/).each do |frame|
+  #   page.frames_with(src: /foo/).each do |frame|
   #     p frame.src
   #   end
 
@@ -330,14 +418,22 @@ class Mechanize::Page < Mechanize::File
   ##
   # :method: iframe_with
   #
-  # :call-seq: iframe_with(criteria)
+  # :call-seq:
+  #   iframe_with(criteria)
+  #   iframe_with(criteria) { |iframe| ... }
+  #
+  # Find a single iframe tag matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "iframe tag(s)".
   #
-  # Find a single iframe tag matching +criteria+.
   # Example:
-  #   page.iframe_with(:src => /foo/).click
+  #   page.iframe_with(src: /foo/).click
 
   ##
-  # :mehtod: iframe_with!(criteria)
+  # :method: iframe_with!
+  #
+  # :call-seq:
+  #   iframe_with!(criteria)
+  #   iframe_with!(criteria) { |iframe| ... }
   #
   # Same as +iframe_with+ but raises an ElementNotFoundError if no button
   # matches +criteria+
@@ -347,9 +443,11 @@ class Mechanize::Page < Mechanize::File
   #
   # :call-seq: iframes_with(criteria)
   #
-  # Find all iframe tags matching +criteria+.
+  # Find all iframe tags matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "iframe tag(s)".
+  #
   # Example:
-  #   page.iframes_with(:src => /foo/).each do |iframe|
+  #   page.iframes_with(src: /foo/).each do |iframe|
   #     p iframe.src
   #   end
 
@@ -358,14 +456,22 @@ class Mechanize::Page < Mechanize::File
   ##
   # :method: image_with
   #
-  # :call-seq: image_with(criteria)
+  # :call-seq:
+  #   image_with(criteria)
+  #   image_with(criteria) { |image| ... }
+  #
+  # Find a single image matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "image(s)".
   #
-  # Find a single image matching +criteria+.
   # Example:
-  #   page.image_with(:alt => /main/).fetch.save
+  #   page.image_with(alt: /main/).fetch.save
 
   ##
-  # :mehtod: image_with!(criteria)
+  # :method: image_with!
+  #
+  # :call-seq:
+  #   image_with!(criteria)
+  #   image_with!(criteria) { |image| ... }
   #
   # Same as +image_with+ but raises an ElementNotFoundError if no button matches
   # +criteria+
@@ -375,9 +481,11 @@ class Mechanize::Page < Mechanize::File
   #
   # :call-seq: images_with(criteria)
   #
-  # Find all images matching +criteria+.
+  # Find all images matching +criteria+.  See +forms_with+ for
+  # details of +criteria+, where for "form(s)" read "image(s)".
+  #
   # Example:
-  #   page.images_with(:src => /jpg\Z/).each do |img|
+  #   page.images_with(src: /jpg\Z/).each do |img|
   #     img.fetch.save
   #   end
 
