@@ -2,6 +2,7 @@ require 'mechanize/file'
 require 'mechanize/file_saver'
 require 'mechanize/page'
 require 'mechanize/xml_file'
+require 'mime/types'
 
 ##
 # Mechanize allows different parsers for different content types.  Mechanize
@@ -68,6 +69,15 @@ class Mechanize::PluggableParser
     :xml   => ['text/xml', 'application/xml'],
   }
 
+  InvalidContentTypeError =
+    if defined?(MIME::Type::InvalidContentType)
+      # For mime-types >=2.1
+      MIME::Type::InvalidContentType
+    else
+      # For mime-types <2.1
+      MIME::InvalidContentType
+    end
+
   attr_accessor :default
 
   def initialize
@@ -97,9 +107,11 @@ class Mechanize::PluggableParser
 
     parser = @parsers[mime_type.to_s] ||
              @parsers[mime_type.simplified] ||
+             # Starting from mime-types 3.0 x-prefix is deprecated as per IANA
+             (@parsers[MIME::Type.simplified(mime_type.to_s, remove_x_prefix: true)] rescue nil) ||
              @parsers[mime_type.media_type] ||
              default
-  rescue MIME::Type::InvalidContentType
+  rescue InvalidContentTypeError
     default
   end
 
