@@ -1,6 +1,8 @@
 class Mechanize::UnsupportedSchemeError < Mechanize::Error
-  attr_accessor :scheme
-  def initialize(scheme)
+  attr_accessor :scheme, :uri
+
+  def initialize(scheme, uri)
     @scheme = scheme
+    @uri    = uri
   end
 end
