@@ -14,7 +14,10 @@ class TestMechanizeHttpAuthRealm < Mechanize::TestCase
     assert_equal 'r', @realm.realm
 
     realm = @AR.new 'Digest', @uri, 'R'
-    assert_equal 'r', realm.realm
+    refute_equal 'r', realm.realm
+
+    realm = @AR.new 'Digest', @uri, 'R'
+    assert_equal 'R', realm.realm
 
     realm = @AR.new 'Digest', @uri, nil
     assert_nil realm.realm
@@ -30,6 +33,9 @@ class TestMechanizeHttpAuthRealm < Mechanize::TestCase
     other = @AR.new 'Digest', URI('http://other.example/'), 'r'
     refute_equal @realm, other
 
+    other = @AR.new 'Digest', @uri, 'R'
+    refute_equal @realm, other
+
     other = @AR.new 'Digest', @uri, 's'
     refute_equal @realm, other
   end
