@@ -6,16 +6,10 @@ class TestMechanizeUtil < Mechanize::TestCase
 
   INPUTTED_VALUE = "テスト" # "test" in Japanese UTF-8 encoding
   CONTENT_ENCODING = 'Shift_JIS' # one of Japanese encoding
-  ENCODED_VALUE = "\x83\x65\x83\x58\x83\x67" # "test" in Japanese Shift_JIS encoding
+  ENCODED_VALUE = "\x83\x65\x83\x58\x83\x67".force_encoding(::Encoding::SHIFT_JIS) # "test" in Japanese Shift_JIS encoding
 
-  if Mechanize::Util::NEW_RUBY_ENCODING
-    ENCODING_ERRORS = [EncodingError, Encoding::ConverterNotFoundError] # and so on
-    ERROR_LOG_MESSAGE = /from_native_charset: Encoding::ConverterNotFoundError: form encoding: "UTF-eight"/
-    ENCODED_VALUE.force_encoding(::Encoding::SHIFT_JIS)
-  else
-    ENCODING_ERRORS = [Iconv::InvalidEncoding, Iconv::IllegalSequence]
-    ERROR_LOG_MESSAGE = /from_native_charset: Iconv::InvalidEncoding: form encoding: "UTF-eight"/
-  end
+  ENCODING_ERRORS = [EncodingError, Encoding::ConverterNotFoundError] # and so on
+  ERROR_LOG_MESSAGE = /from_native_charset: Encoding::ConverterNotFoundError: form encoding: "UTF-eight"/
 
   INVALID_ENCODING = 'UTF-eight'
 
@@ -104,5 +98,46 @@ class TestMechanizeUtil < Mechanize::TestCase
     assert_equal "%",   @MU.uri_escape("%", /[^%]/)
   end
 
+  def test_build_query_string_simple
+    input_params = [
+      [:ids, 1],
+      [:action, 'delete'],
+      [:ids, 5],
+    ]
+
+    expected_params = [
+      ['ids', '1'],
+      ['action', 'delete'],
+      ['ids', '5'],
+    ]
+
+    query = @MU.build_query_string(input_params)
+
+    assert_equal expected_params, URI.decode_www_form(query)
+  end
+
+  def test_build_query_string_complex
+    input_params = {
+      number: 7,
+      name: "\u{6B66}\u{8005}",
+      "ids[]" => [1, 3, 5, 7],
+      words: ["Sing", "Now!"],
+      params: { x: "50%", y: "100%", t: [80, 160] },
+    }
+
+    expected_params = [
+      ['number', '7'],
+      ['name', "\u{6B66}\u{8005}"],
+      ['ids[]', '1'], ['ids[]', '3'], ['ids[]', '5'], ['ids[]', '7'],
+      ['words', 'Sing'], ['words', 'Now!'],
+      ['params[x]', '50%'],
+      ['params[y]', '100%'],
+      ['params[t]', '80'], ['params[t]', '160'],
+    ]
+
+    query = @MU.build_query_string(input_params)
+
+    assert_equal expected_params, URI.decode_www_form(query)
+  end
 end
 
